@@ -37,6 +37,8 @@ use Lintian::Util qw(internal_error lstrip rstrip);
 
 with('Lintian::Check');
 
+has timers => (is => 'rwp', default => sub{ [] });
+
 # Init scripts that do not need a service file
 my $INIT_WHITELIST = Lintian::Data->new('systemd/init-whitelist');
 
@@ -46,6 +48,25 @@ my $HARDENING_FLAGS = Lintian::Data->new('systemd/hardening-flags');
 # Usual WantedBy= targets
 my $WANTEDBY_WHITELIST = Lintian::Data->new('systemd/wantedby-whitelist');
 
+sub setup {
+    my ($self) = @_;
+
+    my @timers = grep { m,^lib/systemd/system/[^\/]+\.timer$, }
+      $self->info->sorted_index;
+    $self->_set_timers(\@timers);
+
+    return;
+}
+
+sub files {
+    my ($self, $file) = @_;
+
+    $self->tag('missing-systemd-timer-for-cron-script', $file)
+      if $file->dirname =~ m,^etc/cron\.[^\/]+/$, && !scalar @{$self->timers};
+
+    return;
+}
+
 sub binary {
     my ($self) = @_;
 
@@ -74,7 +95,6 @@ sub binary {
         $self->check_init_script($script, $services);
     }
 
-    $self->check_timers();
     $self->check_maintainer_scripts();
 
     return;
@@ -415,41 +435,26 @@ sub extract_service_file_values {
     return @values;
 }
 
-sub check_timers {
-    my ($self) = @_;
-
-    my $info = $self->info;
-
-    return
-      if any { m,^lib/systemd/system/[^\/]+\.timer$, } $info->sorted_index;
-
-    for my $file ($info->sorted_index) {
-        $self->tag('missing-systemd-timer-for-cron-script', $file)
-          if $file->dirname =~ m,^etc/cron\.[^\/]+/$,;
-    }
-
-    return;
-}
-
 sub check_maintainer_scripts {
     my ($self) = @_;
 
     my $info = $self->info;
 
-    open(my $fd, '<', $info->lab_data_path('control-scripts'));
+    # get maintainer scripts
+    my %control = %{$self->info->control_scripts};
+
+    for my $file (keys %control) {
 
-    while (<$fd>) {
-        m/^(\S*) (.*)$/
-          or internal_error("bad line in control-scripts file: $_");
-        my $interpreter = $1;
-        my $file = $2;
         my $path = $info->control_index_resolved_path($file);
+        my $interpreter = $control{$file};
 
         # Don't follow unsafe links
-        next if not $path or not $path->is_open_ok;
-        # Don't try to parse the file if it does not appear to be a
-        # shell script
-        next if $interpreter !~ m/sh\b/;
+        next
+          unless $path && $path->is_open_ok;
+
+        # skip anything but shell scripts
+        next
+          unless $interpreter =~ m/sh\b/;
 
         my $sfd = $path->open;
         while (<$sfd>) {
@@ -465,7 +470,6 @@ sub check_maintainer_scripts {
         close($sfd);
     }
 
-    close($fd);
     return;
 }
 
