@@ -244,6 +244,15 @@ sub check_test_file {
         );
     } elsif (not $index->is_open_ok) {
         $self->tag('runtime-test-file-is-not-a-regular-file', $path);
+    } else {
+        my $fd = $index->open;
+        while (my $x = <$fd>) {
+            if ($x =~ m/ADTTMP/) {
+                $self->tag('uses-deprecated-adttmp', $path, "(line $.)");
+                last;
+            }
+        }
+        close($fd);
     }
     # Test files are allowed not to be executable.
     return;
