@@ -25,13 +25,18 @@ use warnings;
 use autodie;
 use parent 'Lintian::Collect::Package';
 
+use BerkeleyDB;
 use Carp qw(croak);
+use List::Util qw(any);
+use MLDBM qw(BerkeleyDB::Btree Storable);
 use Path::Tiny;
 
 use Lintian::Deb822Parser qw(parse_dpkg_control);
 use Lintian::Info::Changelog;
 use Lintian::Relation;
-use Lintian::Util qw(internal_error open_gz get_file_checksum strip);
+use Lintian::Util qw(open_gz get_file_checksum strip rstrip);
+
+use constant EMPTY => q{};
 
 =head1 NAME
 
@@ -41,9 +46,6 @@ Lintian::Collect::Binary - Lintian interface to binary package data collection
 
     my ($name, $type, $dir) = ('foobar', 'binary', '/path/to/lab-entry');
     my $collect = Lintian::Collect->new ($name, $type, $dir);
-    if ($collect->native) {
-        print "Package is native\n";
-    }
 
 =head1 DESCRIPTION
 
@@ -58,6 +60,8 @@ binary packages via this module so that the module can cache data where
 appropriate and possibly retire collect scripts in favor of caching that
 data in memory.
 
+Native heuristics are only available in source packages.
+
 =head1 CLASS METHODS
 
 =over 4
@@ -87,38 +91,6 @@ L<Lintian::Collect::Package> modules are also available.
 
 =over 4
 
-=item native
-
-Returns true if the binary package is native and false otherwise.
-Nativeness will be judged by the source version number.
-
-If the version number is absent, this will return false (as
-native packages are a lot rarer than non-native ones).
-
-Needs-Info requirements for using I<native>: L<Same as field|Lintian::Collect/field ([FIELD[, DEFAULT]])>
-
-=cut
-
-sub native {
-    my ($self) = @_;
-    return $self->{native} if exists $self->{native};
-    my $version;
-    my $source = $self->field('source');
-    if (defined $source && $source =~ m/\((.*)\)/) {
-        $version = $1;
-    } else {
-        $version = $self->field('version');
-    }
-    if (defined $version) {
-        $self->{native} = ($version !~ m/-/);
-    } else {
-        # We do not know, but assume it to non-native as it is
-        # the most likely case.
-        $self->{native} = 0;
-    }
-    return $self->{native};
-}
-
 =item changelog
 
 Returns the changelog of the binary package as a Parse::DebianChangelog
@@ -335,31 +307,46 @@ Needs-Info requirements for using I<scripts>: scripts
 
 sub scripts {
     my ($self) = @_;
-    return $self->{scripts} if exists $self->{scripts};
-    my $scrf = $self->lab_data_path('scripts');
-    my %scripts;
-    if (-f $scrf) {
-        open(my $fd, '<', $scrf);
-        while (my $line = <$fd>) {
-            my (%file, $name);
-            chomp($line);
 
-            $line =~ m/^(env )?(\S*) (.*)$/o
-              or internal_error("bad line in scripts file: $line");
-            ($file{calls_env}, $file{interpreter}, $name) = ($1, $2, $3);
+    unless (exists $self->{scripts}) {
 
-            $name =~ s,^\./,,o;
-            $name =~ s,/+$,,o;
-            $file{name} = $name;
-            $scripts{$name} = \%file;
-        }
-        close($fd);
+        my %scripts;
+
+        my $dbpath = $self->lab_data_path('scripts.db');
+
+        tie my %h, 'MLDBM',-Filename => $dbpath
+          or die "Cannot open file $dbpath: $! $BerkeleyDB::Error\n";
+
+        $scripts{$_} = $h{$_} for keys %h;
+
+        untie %h;
+
+        $self->{scripts} = \%scripts;
     }
-    $self->{scripts} = \%scripts;
 
     return $self->{scripts};
 }
 
+=item is_script (PATH)
+
+True if PATH is a script.
+
+Needs-Info requirements for using I<is_script>: scripts
+
+=cut
+
+sub is_script {
+    my ($self, $path) = @_;
+
+    return 0
+      unless defined $path;
+
+    return 1
+      if exists $self->scripts->{$path};
+
+    return 0;
+}
+
 =item objdump_info
 
 Returns a hashref mapping a FILE to the data collected by objdump-info
@@ -688,11 +675,11 @@ Needs-Info requirements for using I<is_pkg_class>: L<Same as field|Lintian::Coll
         $pkg_class //= 'any-meta';
         if ($pkg_class eq 'debug') {
             return 1 if $self->name =~ m/-dbg(?:sym)?/;
-            return;
+            return 0;
         }
         if ($pkg_class eq 'auto-generated') {
             return 1 if $self->field('auto-built-package');
-            return;
+            return 0;
         }
         return 1 if $desc =~ m/transitional package/;
         $desc = lc($desc);
@@ -703,10 +690,52 @@ Needs-Info requirements for using I<is_pkg_class>: L<Same as field|Lintian::Coll
             return 1 if $section =~ m,(?:^|/)(?:tasks|metapackages)$,;
             return 1 if $self->name =~ m/^task-/;
         }
-        return;
+        return 0;
     }
 }
 
+=item conffiles
+
+Returns a list of absolute filenames found for conffiles.
+
+Needs-Info requirements for using I<conffiles>: L<Same as control_index_resolved_path|/control_index_resolved_path(PATH)>
+
+=cut
+
+sub conffiles {
+    my ($self) = @_;
+
+    return @{$self->{'conffiles'}}
+      if exists $self->{'conffiles'};
+
+    $self->{'conffiles'} = [];
+
+    # read conffiles if it exists and is a file
+    my $cf = $self->control_index_resolved_path('conffiles');
+    return
+      unless $cf && $cf->is_file && $cf->is_open_ok;
+
+    my $fd = $cf->open;
+    while (my $absolute = <$fd>) {
+
+        chomp $absolute;
+
+        # dpkg strips whitespace (using isspace) from the right hand
+        # side of the file name.
+        rstrip($absolute);
+
+        next
+          if $absolute eq EMPTY;
+
+        # list contains absolute paths, unlike lookup
+        push(@{$self->{conffiles}}, $absolute);
+    }
+
+    close($fd);
+
+    return @{$self->{conffiles}};
+}
+
 =item is_conffile (FILE)
 
 Returns a truth value if FILE is listed in the conffiles control file.
@@ -724,26 +753,25 @@ Needs-Info requirements for using I<is_conffile>: L<Same as control_index_resolv
 
 sub is_conffile {
     my ($self, $file) = @_;
-    if (exists $self->{'conffiles'}) {
-        return 1 if exists $self->{'conffiles'}{$file};
-        return;
+
+    unless (exists $self->{'conffiles_lookup'}) {
+
+        $self->{'conffiles_lookup'} = {};
+
+        for my $absolute ($self->conffiles) {
+
+            # strip the leading slash
+            my $relative = $absolute;
+            $relative =~ s,^/++,,;
+
+            # look up happens with a relative path
+            $self->{conffiles_lookup}{$relative} = 1;
+        }
     }
-    my $cf = $self->control_index_resolved_path('conffiles');
-    my %conffiles;
-    $self->{'conffiles'} = \%conffiles;
-    return if not $cf or not $cf->is_open_ok;
-    my $fd = $cf->open;
-    while (my $line = <$fd>) {
-        chomp $line;
-        next if $line =~ m/^\s*$/;
-        # Look up happens with a relative path (e.g. etc/file.conf).
-        # Side-effect is that we silently "fix" relative conffiles,
-        # but checks/conffiles catches those for us.
-        $line =~ s,^/++,,o;
-        $conffiles{$line} = 1;
-    }
-    close($fd);
-    return 1 if exists $conffiles{$file};
+
+    return 1
+      if exists $self->{conffiles_lookup}{$file};
+
     return;
 }
 
